@@ -184,7 +184,7 @@ perform_action (GList *names, GdkDragAction action, char *destdir)
 		dest_name = g_concat_dir_and_file (destdir, x_basename (name));
 
 		do {
-			result = mc_stat (name, &s);
+			result = mc_lstat (name, &s);
 
 			if (result != 0) {
 				/* FIXME: this error message sucks */
