@@ -26,11 +26,48 @@
 #include "setup.h"
 #include "../vfs/vfs.h"
 #include "gprefs.h"
-
+#include "listing-iconic.xpm"
+#include "listing-brief-list.xpm"
+#include "listing-list.xpm"
+#include "listing-custom.xpm"
 
 
 #define UNDEFINED_INDEX -1
 
+/* Keep these two arrays in sync! */
+
+GnomeUIInfo panel_view_menu_uiinfo[] = {
+	GNOMEUIINFO_RADIOITEM (N_("_Icon View"),
+			       N_("Switch view to an icon display"),
+			       gnome_icon_view_cmd, NULL),
+	GNOMEUIINFO_RADIOITEM (N_("_Brief View"),
+			       N_("Switch view to show just file name and type"),
+			       gnome_brief_view_cmd, NULL),
+	GNOMEUIINFO_RADIOITEM (N_("_Detailed View"),
+			       N_("Switch view to show detailed file statistics"),
+			       gnome_detailed_view_cmd, NULL),
+	GNOMEUIINFO_RADIOITEM (N_("_Custom View"),
+			       N_("Switch view to show user-defined statistics"),
+			       gnome_custom_view_cmd, NULL),
+	GNOMEUIINFO_END
+};
+
+GnomeUIInfo panel_view_toolbar_uiinfo[] = {
+	GNOMEUIINFO_RADIOITEM (N_("Icons"),
+			       N_("Switch view to an icon display"),
+			       gnome_icon_view_cmd, listing_iconic_xpm),
+	GNOMEUIINFO_RADIOITEM (N_("Brief"),
+			       N_("Switch view to show just file name and type"),
+			       gnome_brief_view_cmd, listing_brief_list_xpm),
+	GNOMEUIINFO_RADIOITEM (N_("Detailed"),
+			       N_("Switch view to show detailed file statistics"),
+			       gnome_detailed_view_cmd, listing_list_xpm),
+	GNOMEUIINFO_RADIOITEM (N_("Custom"),
+			       N_("Switch view to show user-defined statistics"),
+			       gnome_custom_view_cmd, listing_custom_xpm),
+	GNOMEUIINFO_END
+};
+
 GList *containers = 0;
 
 int output_lines = 0;
@@ -88,7 +125,7 @@ get_other_index (void)
 	return UNDEFINED_INDEX;
 }
 
-static void
+void
 set_current_panel (WPanel *panel)
 {
 	GList *p;
@@ -99,6 +136,7 @@ set_current_panel (WPanel *panel)
 	for (p = containers; p; p = p->next){
 		if (((PanelContainer *)p->data)->panel == panel){
 			current_panel_ptr = p->data;
+			break;
 		}
 	}
 }
@@ -115,7 +153,6 @@ assign_other (void)
 	for (p = containers; p; p = p->next)
 		if (p->data != current_panel_ptr){
 			other_panel_ptr = p->data;
-			printf ("PANEL: Found another other\n");
 			break;
 		}
 }
@@ -252,7 +289,8 @@ save_panel_types (void)
 	for (p = containers; p; p = p->next){
 		PanelContainer *pc = p->data;
 
-		panel_save_setup (pc->panel, pc->panel->panel_name);
+		if (!is_a_desktop_panel (pc->panel))
+			panel_save_setup (pc->panel, pc->panel->panel_name);
 	}
 }
 
@@ -276,7 +314,7 @@ gnome_exit (void)
 	w = gnome_message_box_new (
 		_("Notice that if you choose to terminate the file manager, you will\n"
 		  "also terminate the GNOME desktop handler.\n\n"
-		  "Are you sure you want to quit?"),
+		  "Are you sure you want to exit?"),
 		GNOME_MESSAGE_BOX_WARNING,
 		GNOME_STOCK_BUTTON_YES,
 		GNOME_STOCK_BUTTON_NO,
@@ -313,10 +351,10 @@ do_desktop_rescan_devices (void)
 void configure_box (void);
 
 GtkCheckMenuItem *gnome_toggle_snap (void);
-GnomeUIInfo gnome_panel_new_menu [] = {
+static GnomeUIInfo gnome_panel_new_menu [] = {
 	 GNOMEUIINFO_ITEM_NONE(N_("_Terminal"), N_("Launch a new terminal in the current directory"), gnome_open_terminal),
 	/* If this ever changes, make sure you update create_new_menu accordingly. */
-	GNOMEUIINFO_ITEM_NONE( N_("_Directory..."), N_("Creates a new directory"), gnome_mkdir_cmd ),
+	GNOMEUIINFO_ITEM_NONE(N_("_Directory..."), N_("Creates a new directory"), gnome_mkdir_cmd),
 	GNOMEUIINFO_END
 };
 
@@ -335,8 +373,8 @@ GnomeUIInfo gnome_panel_file_menu [] = {
 	/*  etc... */
 	GNOMEUIINFO_MENU_NEW_SUBTREE(gnome_panel_new_menu),
 	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_OPEN_ITEM( gnome_open_files, NULL),
-/*	GNOMEUIINFO_ITEM_NONE( N_("Open _FTP site"),  N_("Opens an FTP site"), ftplink_cmd },*/
+	GNOMEUIINFO_MENU_OPEN_ITEM(gnome_open_files, NULL),
+/*	GNOMEUIINFO_ITEM_NONE(N_("Open _FTP site"),  N_("Opens an FTP site"), ftplink_cmd },*/
 	GNOMEUIINFO_ITEM_STOCK(N_("_Copy..."), N_("Copy files"), copy_cmd, GNOME_STOCK_PIXMAP_COPY),
 	GNOMEUIINFO_ITEM_STOCK(N_("_Delete..."), N_("Delete files"), delete_cmd, GNOME_STOCK_PIXMAP_REMOVE),
         GNOMEUIINFO_ITEM_NONE(N_("_Move..."), N_("Rename or move files"), ren_cmd),
@@ -350,28 +388,20 @@ GnomeUIInfo gnome_panel_file_menu [] = {
 GnomeUIInfo gnome_panel_edit_menu [] = {
 	{ GNOME_APP_UI_ITEM,  N_("Select _All"),        N_("Select all files in the current Panel"), gnome_select_all_cmd,
 	  NULL, NULL, 0, NULL, 'a', GDK_CONTROL_MASK  },
-	GNOMEUIINFO_ITEM_NONE( N_("_Select Files..."),  N_("Select a group of files"), select_cmd ),
-	GNOMEUIINFO_ITEM_NONE( N_("_Invert Selection"), N_("Reverses the list of tagged files"), reverse_selection_cmd ),
+	GNOMEUIINFO_ITEM_NONE(N_("_Select Files..."),  N_("Select a group of files"), select_cmd),
+	GNOMEUIINFO_ITEM_NONE(N_("_Invert Selection"), N_("Reverses the list of tagged files"), reverse_selection_cmd),
 	GNOMEUIINFO_SEPARATOR,
-        GNOMEUIINFO_ITEM_NONE( N_("_Rescan Directory"), N_("Rescan the directory contents"), reread_cmd ),
+        GNOMEUIINFO_ITEM_NONE(N_("_Rescan Directory"), N_("Rescan the directory contents"), reread_cmd),
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_MENU_PREFERENCES_ITEM(gnome_configure_box, NULL),
 	GNOMEUIINFO_END
 };
 
-GnomeUIInfo gnome_panel_view_menu [] = {
-	GNOMEUIINFO_RADIOITEM(N_("Icon View"), NULL, gnome_icon_view_cmd,NULL),
-	GNOMEUIINFO_RADIOITEM(N_("Partial View"), NULL, gnome_partial_view_cmd,NULL),
-	GNOMEUIINFO_RADIOITEM(N_("Full View"), NULL, gnome_full_view_cmd,NULL),
-	GNOMEUIINFO_RADIOITEM(N_("Custom View"), NULL, gnome_custom_view_cmd,NULL),
-	GNOMEUIINFO_END
-};
-
 GnomeUIInfo gnome_panel_layout_menu [] = {
-	GNOMEUIINFO_ITEM_NONE( N_("_Sort By..."),     N_("Confirmation settings"), gnome_sort_cmd ),
-	GNOMEUIINFO_ITEM_NONE( N_("_Filter View..."),    N_("Global option settings"), gnome_filter_cmd ),
+	GNOMEUIINFO_ITEM_NONE(N_("_Sort By..."),     N_("Confirmation settings"), gnome_sort_cmd),
+	GNOMEUIINFO_ITEM_NONE(N_("_Filter View..."),    N_("Global option settings"), gnome_filter_cmd),
 	GNOMEUIINFO_SEPARATOR,
-        GNOMEUIINFO_RADIOLIST( gnome_panel_view_menu),
+        GNOMEUIINFO_RADIOLIST(panel_view_menu_uiinfo),
 	GNOMEUIINFO_END
 };
 
@@ -388,10 +418,10 @@ GnomeUIInfo gnome_panel_commands_menu [] = {
 #endif
 #ifdef USE_EXT2FSLIB
 	/*does this do anything?*/
-/*	 GNOMEUIINFO_ITEM_NONE(N_("_Undelete files (ext2fs only)..."), N_("Recover deleted files"), undelete_cmd ),*/
+/*	 GNOMEUIINFO_ITEM_NONE(N_("_Undelete files (ext2fs only)..."), N_("Recover deleted files"), undelete_cmd),*/
 #endif
 #ifdef WITH_BACKGROUND
-	GNOMEUIINFO_ITEM_NONE( N_("_Background jobs..."),   N_("List of background operations"), jobs_cmd ),
+	GNOMEUIINFO_ITEM_NONE(N_("_Background jobs..."),   N_("List of background operations"), jobs_cmd),
 #endif
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_ITEM_STOCK (N_("Exit"), N_("Terminates the file manager and the desktop"),
@@ -402,7 +432,7 @@ GnomeUIInfo gnome_panel_commands_menu [] = {
 
 GnomeUIInfo gnome_panel_about_menu [] = {
 /*	GNOMEUIINFO_HELP ("midnight-commander"), */
-	GNOMEUIINFO_MENU_ABOUT_ITEM( gnome_about_cmd,NULL),
+	GNOMEUIINFO_MENU_ABOUT_ITEM(gnome_about_cmd, NULL),
 	GNOMEUIINFO_END
 };
 
@@ -465,30 +495,32 @@ panel_enter_event (GtkWidget *widget, GdkEvent *event, WPanel *panel)
 	send_message (panel->widget.parent, (Widget *) panel, WIDGET_FOCUS, 0);
 }
 
-static void
-create_new_menu_from (GnomeApp *app, WPanel *panel, char *file)
+void
+destroy_gde (GtkWidget *unused, void *data)
+{
+	gnome_desktop_entry_free ((GnomeDesktopEntry *) (data));
+}
+
+gint
+create_new_menu_from (char *file, GtkWidget *shell, gint pos)
 {
-	gint pos;
-	GtkWidget *shell = NULL;
 	DIR *dir;
 	struct stat filedata;
+	gboolean add_separator = TRUE;
 	struct dirent *dirstruc;
 	GnomeDesktopEntry *gde;
 	GtkWidget *menu;
 	char *file2;
-	
+
+	g_return_val_if_fail (shell != NULL, pos);
+
 	dir = opendir (file);
 	if (dir == NULL)
-		return;
-	
-	shell = gnome_app_find_menu_pos (app->menubar, _("File/New/Directory..."), &pos);
-	menu = gtk_menu_item_new ();
-	gtk_widget_show (menu);
-	gtk_menu_shell_insert (GTK_MENU_SHELL (shell), menu, pos++);
+		return pos;
 
 	if (shell == NULL){
 		closedir (dir);
-		return;
+		return pos;
 	}
 
 	while ((dirstruc = readdir (dir)) != NULL){
@@ -499,13 +531,17 @@ create_new_menu_from (GnomeApp *app, WPanel *panel, char *file)
 
 		if ((stat (file2, &filedata) != -1) && (S_ISREG (filedata.st_mode))){
 			char *path;
-			char *test;
-			
+			int len;
+			const int desktoplen = sizeof (".desktop") - 1;
+
+			len = strlen (dirstruc->d_name);
+			if (strcmp (dirstruc->d_name + len - desktoplen, ".desktop") != 0) {
+				g_free (file2);
+				continue;
+			}
+
 			gde = gnome_desktop_entry_load (file2);
-
-			test = strrchr(dirstruc->d_name, '.');
-
-			if (test == NULL || gde == NULL || strcmp (test, ".desktop")){
+			if (gde == NULL) {
 				g_free (file2);
 				continue;
 			}
@@ -513,23 +549,40 @@ create_new_menu_from (GnomeApp *app, WPanel *panel, char *file)
 			path = gnome_is_program_in_path (gde->tryexec);
 			g_free (path);
 			if (!path){
+				gnome_desktop_entry_free (gde);
 				g_free (file2);
 				continue;
 			}
-			
+
+			if (add_separator) {
+				menu = gtk_menu_item_new ();
+				gtk_widget_show (menu);
+				gtk_menu_shell_insert (GTK_MENU_SHELL (shell), menu, pos++);
+				add_separator = !add_separator;
+			}
+
 			menu = gtk_menu_item_new_with_label (gde->name);
 			gtk_widget_show (menu);
 			gtk_menu_shell_insert (GTK_MENU_SHELL (shell), menu, pos++);
 
 			/* This is really bad, but it works. */
+			/* FIXME: it doesn't work if we free the gde below. --
+                         * need to do this right sometime -jrb
+			 */
 			if (gde->comment)
-				gtk_object_set_data (GTK_OBJECT (menu), "apphelper_statusbar_hint",
+				gtk_object_set_data (GTK_OBJECT (menu),
+						     "apphelper_statusbar_hint",
 						     gde->comment);
-			gtk_signal_connect (GTK_OBJECT (menu), "activate", GTK_SIGNAL_FUNC (gnome_run_new),
+			gtk_signal_connect (GTK_OBJECT (menu), "activate",
+					    GTK_SIGNAL_FUNC (gnome_run_new),
+					    gde);
+			gtk_signal_connect (GTK_OBJECT (menu), "destroy",
+					    GTK_SIGNAL_FUNC (destroy_gde),
 					    gde);
 		}
 		g_free (file2);
 	}
+	return pos;
 }
 
 /**
@@ -541,15 +594,20 @@ static void
 create_new_menu (GnomeApp *app, WPanel *panel)
 {
 	gchar *file, *file2;
+	gint pos;
+	GtkWidget *shell;
+
+	shell = gnome_app_find_menu_pos (app->menubar, _("File/New/Directory..."), &pos);
 
 	file = gnome_unconditional_datadir_file ("mc/templates");
-	create_new_menu_from (app, panel, file);
-	
+	pos = create_new_menu_from (file, shell, pos);
+
 	file2 = gnome_datadir_file ("mc/templates");
 	if (file2 != NULL){
 		if (strcmp (file, file2) != 0)
-			create_new_menu_from (app, panel, file2);
+			create_new_menu_from (file2, shell, pos);
 	}
+
 	g_free (file);
 	g_free (file2);
 }
@@ -603,6 +661,37 @@ my_app_create_menus (GnomeApp *app, GnomeUIInfo *uiinfo, void *data)
 	gnome_app_create_menus_custom (app, uiinfo, &uibdata);
 }
 
+/**
+ * copy_uiinfo_widgets:
+ * @uiinfo: A GnomeUIInfo array
+ * @dest: The destination array will be placed here
+ * 
+ * Allocates an array of widgets and copies the widgets from the uiinfo array to
+ * it.  The array will be NULL-terminated.
+ **/
+void
+copy_uiinfo_widgets (GnomeUIInfo *uiinfo, gpointer **dest)
+{
+	int n;
+	int i;
+
+	g_return_if_fail (uiinfo != NULL);
+	g_return_if_fail (dest != NULL);
+
+	/* Count number of items */
+
+	for (n = 0; uiinfo[n].type != GNOME_APP_UI_ENDOFINFO; n++);
+
+	/* Copy the widgets */
+
+	*dest = g_new (gpointer, n + 1);
+
+	for (i = 0; i < n; i++)
+		(*dest)[i] = uiinfo[i].widget;
+
+	(*dest)[i] = NULL;
+}
+
 WPanel *
 create_container (Dlg_head *h, char *name, char *geometry)
 {
@@ -638,8 +727,8 @@ create_container (Dlg_head *h, char *name, char *geometry)
 	else
 		uiinfo = gnome_panel_menu_with_desktop;
 
-
 	my_app_create_menus (GNOME_APP (app), uiinfo, panel);
+	copy_uiinfo_widgets (panel_view_menu_uiinfo, &panel->view_menu_items);
 	
 	create_new_menu (GNOME_APP (app), panel);
 
@@ -718,7 +807,8 @@ update_panels (int force_update, char *current_file)
 	if (!cpanel)
 		return;
 	
-	update_one_panel_widget (cpanel, force_update, current_file);
+	if (!is_a_desktop_panel (cpanel))
+		update_one_panel_widget (cpanel, force_update, current_file);
 
 	if (reload_others){
 		for (p = containers; p; p = p->next){
@@ -727,7 +817,8 @@ update_panels (int force_update, char *current_file)
 			if (p->data == current_panel_ptr)
 				continue;
 			
-			update_one_panel_widget (pc->panel, force_update, UP_KEEPSEL);
+			if (!is_a_desktop_panel (pc->panel))
+				update_one_panel_widget (pc->panel, force_update, UP_KEEPSEL);
 		}
 	}
 	mc_chdir (cpanel->cwd);
