@@ -83,7 +83,7 @@ panel_action_edit (GtkWidget *widget, WPanel *panel)
 {
 	char *full = g_concat_dir_and_file (panel->cwd, selection (panel)->fname);
 	
-	gmc_edit_filename (full);
+	gmc_edit (full);
 	g_free (full);
 }
 
@@ -102,12 +102,6 @@ dicon_properties (GtkWidget *widget, DesktopIconInfo *dii)
 	if (gnome_dialog_run (GNOME_DIALOG (dlg)) == 0)
 		retval = gnome_file_property_dialog_make_changes (GNOME_FILE_PROPERTY_DIALOG (dlg));
 
-	/*
-	 * Apply: desktop icon name change and url change
-	 */
-	if (dii->url)
-		desktop_icon_update_url (dii);
-	
 	gtk_widget_destroy (dlg);
 	g_free(path);
 
@@ -161,7 +155,7 @@ panel_action_properties (GtkWidget *widget, WPanel *panel)
 /*	if (item_properties (GTK_WIDGET (CLIST_FROM_SW (panel->list)), full_name, NULL) != 0)
 	reread_cmd ();*/
 	dlg = gnome_file_property_dialog_new (full_name, we_can_afford_the_speed);
-	gnome_dialog_set_parent (GNOME_DIALOG (dlg), GTK_WINDOW (gtk_widget_get_toplevel (panel->ministatus)));
+	gnome_dialog_set_parent (GNOME_DIALOG (dlg), GTK_WINDOW (panel->xwindow));
 	if (gnome_dialog_run (GNOME_DIALOG (dlg)) == 0)
 		retval = gnome_file_property_dialog_make_changes (GNOME_FILE_PROPERTY_DIALOG (dlg));
 		gtk_widget_destroy (dlg);
@@ -256,6 +250,8 @@ fill_menu (GtkMenuShell *menu_shell, GnomeUIInfo *uiinfo, int pos)
  */
 static struct action file_actions[] = {
 	{ N_("Properties"),      F_SINGLE | F_PANEL,   	  	 (GtkSignalFunc) panel_action_properties },
+	{ N_("Delete"),          F_PANEL | F_ALL,                (GtkSignalFunc) delete_cmd },
+	{ N_("Delete"),          F_DICON | F_ALL,                (GtkSignalFunc) dicon_delete },
 	{ N_("Properties"),      F_SINGLE | F_DICON,  	  	 (GtkSignalFunc) dicon_properties },
 	{ N_("Mount device"),    F_SINGLE|F_MOUNTABLE|F_DICON,   (GtkSignalFunc) dicon_mount },
 	{ N_("Unmount device"),  F_SINGLE|F_UNMOUNTABLE|F_DICON, (GtkSignalFunc) dicon_unmount },
@@ -268,34 +264,15 @@ static struct action file_actions[] = {
 	{ N_("View unfiltered"), F_PANEL | F_NOTDIR,      	 (GtkSignalFunc) panel_action_view_unfiltered },  
 	{ N_("Edit"),            F_PANEL | F_NOTDIR,             (GtkSignalFunc) panel_action_edit },
 	{ "",                    0,          	                 NULL },
+	{ N_("Move/rename..."),  F_PANEL | F_ALL,                (GtkSignalFunc) ren_cmd },
+	{ N_("Copy..."),         F_PANEL | F_ALL,                (GtkSignalFunc) copy_cmd },
+	{ "",                    0,          	                 NULL },
 	{ N_("Link..."),         F_PANEL | F_REGULAR | F_SINGLE, (GtkSignalFunc) link_cmd },
 	{ N_("Symlink..."),      F_PANEL | F_SINGLE,             (GtkSignalFunc) symlink_cmd },
 	{ N_("Edit symlink..."), F_PANEL | F_SYMLINK,            (GtkSignalFunc) edit_symlink_cmd },
 	{ NULL, 0, NULL },
 };
 
-/* Menu entries for files from a panel window */
-static GnomeUIInfo panel_actions[] = {
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_ITEM_NONE (N_("Move/rename..."), NULL, ren_cmd),
-	GNOMEUIINFO_ITEM_NONE (N_("Copy..."), NULL, copy_cmd),
-	GNOMEUIINFO_ITEM_NONE (N_("Delete"), NULL, delete_cmd),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_END
-};
-
-/* Menu entries for files from desktop icons */
-static GnomeUIInfo dicon_actions[] = {
-	GNOMEUIINFO_SEPARATOR,
-#if 0
-	GNOMEUIINFO_ITEM_NONE (N_("Move/rename..."), NULL, dicon_move),
-	GNOMEUIINFO_ITEM_NONE (N_("Copy..."), NULL, dicon_copy),
-#endif
-	GNOMEUIINFO_ITEM_NONE (N_("Delete"), NULL, dicon_delete),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_END
-};
-
 
 /* Creates the menu items for the standard actions.  Returns the position at which additional menu
  * items should be inserted.
@@ -528,7 +505,6 @@ create_regexp_actions (GtkWidget *menu, WPanel *panel,
 		       int panel_row, char *filename, int insert_pos)
 {
 	gpointer closure, regex_closure;
-	GnomeUIInfo *a_uiinfo;
 	int i;
 	GtkSignalFunc regex_callback;
 	const char *mime_type;
@@ -538,26 +514,15 @@ create_regexp_actions (GtkWidget *menu, WPanel *panel,
 		GNOMEUIINFO_END
 	};
 
+	regex_callback = mime_command_from_desktop_icon;
 	if (panel) {
-		a_uiinfo = panel_actions;
 		closure = panel;
-		regex_callback = mime_command_from_desktop_icon;
 		regex_closure = filename;
 	} else {
-		a_uiinfo = dicon_actions;
 		closure = dii;
-		regex_callback = mime_command_from_desktop_icon;
 		regex_closure = filename;
 	}
 
-	/* Fill in the common part of the menus */
-
-	for (i = 0; a_uiinfo [i].type != GNOME_APP_UI_ENDOFINFO; i++)
-		a_uiinfo[i].user_data = closure;
-
-	fill_menu (GTK_MENU_SHELL (menu), a_uiinfo, insert_pos);
-	insert_pos += 5; /* the number of items from the common menus */
-
 	/* Fill in the regex command part */
 
 	mime_type = gnome_mime_type_or_default (filename, NULL);
