@@ -454,7 +454,12 @@ void init_subshell (void)
 	    break;
 
 	    case ZSH:
-	    execl (shell, "zsh", "+Z", NULL);
+	    /* change from "+Z" to "-Z" by Michael Bramer
+	     * (Debian-mc-maintainer) <grisu@debian.org> from a patch from
+	     * Radovan Garabik <garabik@center.fmph.uniba.sk>
+	     */
+	    execl (shell, "zsh", "-Z", NULL);
+
 	    break;
 	}
 
@@ -1115,6 +1120,7 @@ static int pty_open_slave (const char *pty_name)
     }
 
 #if !defined(__osf__)
+#if defined (I_FIND) && defined (I_PUSH)
     if (!ioctl (pty_slave, I_FIND, "ptem"))
 	if (ioctl (pty_slave, I_PUSH, "ptem") == -1)
 	{
@@ -1140,6 +1146,7 @@ static int pty_open_slave (const char *pty_name)
 	    return -1;
 	}
 #endif /* sgi || __sgi */
+#endif /* I_FIND && I_PUSH */
 #endif /* __osf__ */
 
     return pty_slave;
