@@ -76,7 +76,7 @@
 #include "user.h"		/* expand_format */
 #include "../vfs/vfs.h"
 
-/* "$Id: util.c,v 1.20 1999/03/02 21:49:04 norbert Exp $" */
+/* "$Id: util.c,v 1.21 1999/03/10 21:56:21 norbert Exp $" */
 
 char app_text [] = "Midnight-Commander";
 
@@ -661,9 +661,8 @@ char *x_basename (char *s)
 void my_putenv (char *name, char *data)
 {
     char *full;
-    char *len = strlen (name) + strlen (data) + 2;
 
-    full = malloc (len);
+    full = malloc (strlen (name) + strlen (data) + 2);
     strcpy (full, name);
     strcat (full, "=");
     strcat (full, data);
