@@ -121,7 +121,7 @@ int altered_magic_flag = 0;
 int altered_nroff_flag = 0;
 /* }}} */
 
-/* "$Id: view.c,v 1.28 1999/02/26 07:26:45 unammx Exp $" */
+/* "$Id: view.c,v 1.29 1999/03/11 02:41:26 unammx Exp $" */
 
 static char hex_char[] = "0123456789ABCDEF";
 
@@ -1441,7 +1441,9 @@ search (WView *view, char *text, int (*search)(WView *, char *, char *, int))
     Dlg_head *d = 0;
     int search_status;
     int abort;
-    void *gd;
+#ifdef HAVE_GNOME
+    GtkWidget *gd;
+#endif
     
     /* Used to keep track of where the line starts, when looking forward */
     /* is the index before transfering the line; the reverse case uses   */
@@ -1833,13 +1835,6 @@ toggle_hex_mode (WView *view)
     view_update (view, TRUE);
 }
 
-/* Both views */
-static void
-toggle_hexedit_mode(WView *view)
-{
-    view->hexedit_mode = 1 - view->hexedit_mode;
-}
-
 /* Both views */
 void
 goto_line (WView *view)
